/*
   A simple example of communication from the Launchpad to the computer:
  the value of analog input 0 is sent out the serial port. We call this "serial"
  communication because the connection appears to both the Launchpad and the
  computer as a serial port, even though it may actually use
  a USB cable. Bytes are sent one after another (serially) from the Launchpad
  to the computer.

  You can use the Energia serial monitor to view the sent data, or it can
  be read by Processing or another program capable of reading
  data from a serial port. The Processing code below graphs the data received
  so you can see the value of the analog input changing over time.

  The circuit:
  Any analog input sensor can be attached to an analog pin.
  For this example, a potentiometer is attached to analog pin 3.

  created 2006
  by David A. Mellis
  modified 9 Apr 2012
  by Tom Igoe and Scott Fitzgerald
  Modified 15 April 2013
  By Sean Alvarado
  Modified  July 19, 2017
  by Brian Cruickshank - Serial I/O interface to GCv2

  Hardware Required:
 * MSP-EXP430F5529 LaunchPad

  This example code is in the public domain.
 */

#define NUMBER 200
#define MAX_JSON_STRLEN 40
#define NUM_VALUES 10
int greenLedState = HIGH;
int redLedState = LOW;
uint8_t i = 0;
uint32_t average = 0;
uint32_t values[NUM_VALUES];
char jsonStrFromHost[MAX_JSON_STRLEN];
char badStrFromHost[MAX_JSON_STRLEN];
#define FARENHEIT 0
#define CELSIUS 1
uint8_t numBytesRead = 0;
uint8_t j = 0;
uint8_t n = 0;
int rxStrIndex = 0;
int badStrIndex = 0;
int numBadStrings = 0;
boolean toggle = false;
boolean cmdReceived = false;
boolean flag = true;
boolean running = true;
boolean msgOk = false;
int count = 0;
int debounceCounter = 0;
// From http://forum.43oh.com/topic/4730-using-the-internal-temp-sensor-on-f5529/
static const uint16_t *ADC15TEMP30 = (uint16_t *)0x1A1A;
static const uint16_t *ADC15TEMP85 = (uint16_t *)0x1A1C;
static const uint16_t *ADC25TEMP30 = (uint16_t *)0x1A22;
static const uint16_t *ADC25TEMP85 = (uint16_t *)0x1A24;
void setup() {
    pinMode(RED_LED, OUTPUT);
    pinMode(GREEN_LED, OUTPUT);
    analogReference(INTERNAL1V5);
    analogRead(TEMPSENSOR); // first reading usually wrong
    Serial.setTimeout(6000); // 1s timeout when reading from serial port
    Serial.begin(9600);
    pinMode(PUSH2, INPUT_PULLUP);
    greenLedState = HIGH;
    redLedState = LOW;
    digitalWrite(GREEN_LED, greenLedState);
    digitalWrite(RED_LED, redLedState);
    average = 0;

    for (j=0; j<NUM_VALUES; j++) {
        values[j]= getTemperatureTimes100(FARENHEIT);
        average += values[j];
    }
    clearStr(jsonStrFromHost);
    clearStr(badStrFromHost);
    rxStrIndex = 0;

    Serial.flush();
    j=0;
}

void printDec(uint32_t ui) {
    if (ui < 9999){
        Serial.print(ui/100, DEC);
        Serial.print(".");
        if ((ui%100) < 10){
            Serial.print("0");
        }
        Serial.print(ui%100, DEC);
    }
    else {
        Serial.print("99.99");
    }
}
// For debugging purposes: if you get bad strings from the host,
// the last bad string will be copied into the badStrFromHost character array
void copyBadStr(){
    int m = 0;
    clearStr(badStrFromHost);

    for (m=0; m < MAX_JSON_STRLEN; m++){
        badStrFromHost[m] = jsonStrFromHost[m];
    }
    badStrIndex = rxStrIndex;
    rxStrIndex = 0;
}

// From http://forum.43oh.com/topic/4730-using-the-internal-temp-sensor-on-f5529/
int16_t getTemperatureTimes100(int scale)
{
    uint32_t reading=0;
    int16_t result = 0;
    analogReference(INTERNAL1V5); //use the 1.5V internal reference

    reading = analogRead(TEMPSENSOR);
    if (scale == CELSIUS) {
        result = map(reading,*ADC15TEMP30,*ADC15TEMP85,300,850); //convert our reading to Celsius & return
    } else {
        // Fahrenheit calculations:
        result = map(reading,*ADC15TEMP30,*ADC15TEMP85,8600,18500); //convert our reading to Farenheit & return
    }
    return (result);
}

void clearStr(char* pBuffer){
    int k = 0;
    for (k=0; k < MAX_JSON_STRLEN; k++){
        pBuffer[k] = (char)0;
    }
}
int getSerialString(){
    return 0;
    char rxChar;
    boolean done = false;
    int waitCtr = 0;
    int numCharsRx = 0;
    int numAvailable = 0;
    int index = 0;
    while (!done){
        numAvailable = Serial.available();
        if (numAvailable > 0) {
            for (index=0; index < numAvailable; index++){
                waitCtr = 0;
                rxChar = Serial.read();
                if ((rxChar != -1) && (rxChar != (char)0)){
                    if (rxChar == (char)10){
                        done = true;
                        numCharsRx = rxStrIndex;
                        rxStrIndex = 0;
                    } else {
                        if (rxStrIndex < MAX_JSON_STRLEN){
                            jsonStrFromHost[rxStrIndex++] = rxChar;
                        } else {
                            done = true;
                            break;
                        }
                    }
                }
            }
            Serial.flush();
        } else {
            if (rxStrIndex == 0) {
                done = true;
                Serial.flush();
            } else {
                // If a partial command has been received
                // keep waiting until the /n delimiter is received
                // or we timeout
                delayMicroseconds(50);     // pauses for 0.05 milliseconds
                waitCtr++;
                if (waitCtr > 2000) {  // timeout after 1 second
                    clearStr(jsonStrFromHost);
                    rxStrIndex = 0;
                    numCharsRx = 0;
                    done = true;
                    Serial.flush();
                }
            }
        }
    }
    return(numCharsRx);

}
void loop() {
    if (running){


       numBytesRead = getSerialString();//Serial.readBytesUntil('\n',jsonStrFromHost,MAX_JSON_STRLEN);
        if (numBytesRead > 2){
            // Data from the host needs to be of the form {"LEDs":{"Red":0,"Green":0}} to turn off
            // the red and green LEDs, {"LEDs":{"Red":1,"Green":1}} to turn on the red and green LEDs.
            // (Code is not generic at all, but it is small  )
            msgOk = false;

            for (int j=2; j < numBytesRead; j++){
                if ((jsonStrFromHost[j] == ':') && (j < (numBytesRead + 1))){
                    if (jsonStrFromHost[j-2] == 'd'){
                        msgOk = true;
                        if (jsonStrFromHost[j+1] == '1'){
                            redLedState = HIGH;
                        }  else {
                            redLedState = LOW;
                        }
                        digitalWrite(RED_LED, redLedState);
                        cmdReceived = true;
                        flag=true;
                    } else {
                        if (jsonStrFromHost[j-2] == 'n'){
                            msgOk = true;
                            if (jsonStrFromHost[j+1] == '1'){
                                greenLedState = HIGH;
                            }  else {
                                greenLedState = LOW;
                            }
                            digitalWrite(GREEN_LED, greenLedState);
                            cmdReceived = true;
                            flag=true;
                        }
                    }
                }
            }
            if (!msgOk){
                // put a breakpoint here
                numBadStrings++;
                copyBadStr();
            } else {
                clearStr(jsonStrFromHost);
                rxStrIndex = 0;
                flag = true;
            }

        } else {
            if (numBytesRead > 0) {
                numBadStrings++;
                copyBadStr();
            }
        }

        // Formula: http://www.43oh.com/forum/viewtopic.php?p=18248#p18248
        average -= values[n%NUM_VALUES];
        values[n%NUM_VALUES] = (uint32_t)getTemperatureTimes100(FARENHEIT);
        average += values[n%NUM_VALUES];
        count++;
        if (count > 100) {
          count = 0;
        }

        // Print average as a JSON string of the form {"analog":[99.99,0,1]}\n
        if (flag) {
           Serial.print("{\"analog\":[");
            printDec(average/NUM_VALUES);
            Serial.print(",");
            Serial.print(count, DEC);
            Serial.print(",");
            Serial.print((100 - count), DEC);
            Serial.print("]}\n");
            flag = false;
            n = 0;
        }

    }  else {
        // if not running, blink the red LED.
        if (flag) {
            digitalWrite(RED_LED, toggle);
            toggle = !toggle;
            flag = false;
            n = 0;
        }
    }

    n++;
    if (n==NUMBER) {
        flag=true;
        n = 0;
    }
    if (digitalRead(PUSH2)==LOW) {
        if (debounceCounter == 2){
            running = !running;
            if (running) {
                greenLedState = HIGH;
                redLedState = LOW;
            } else {
                greenLedState = LOW;
                redLedState = LOW;
            }
            digitalWrite(GREEN_LED, greenLedState);
            digitalWrite(RED_LED, redLedState);
            debounceCounter = 9;
        } else {
            debounceCounter++;
        }
    } else {
        debounceCounter = 0;
    }

    delay(1);
}



